/******************************************************************************
 * ConnectN.java
 * CREATED: 05/21/2004
 * SYNOPSIS: 
 *****************************************************************************/
import java.io.*;
import java.awt.*;

class ConnectN {

	// global gui color
	public final static Color GUICOLOR = new Color(246,246,255);
	public final static Color TEXTAREACOLOR = new Color(255,255,255);

    // game states - in a game or at startup.
    public final static int STARTUP = 0; // before a game.
    public final static int INGAME = 1;  // in the middle of a game
    public final static int OUTGAME = 2;
    public final static int WINGAME = 3; // after a game - someone won.
    public final static int DRAWGAME = 4; // game is a draw

    // player states. the player turn states match the player numbers for
    // in-game comparison. 
    public final static int PLAYER_ONE_TURN = 5;
    public final static int PLAYER_TWO_TURN = 6;
    public final static int PLAYER_ONE = 5;
    public final static int PLAYER_TWO = 6;
    
    // online states
    public final static int ONLINE = 7;
    public final static int OFFLINE = 8;
    public final static int WAITING = 9; // user has chosen new network game,
                                         // we're waiting for other player.
    
    private int gameState;   // whether we're in-game or not
    private int playerState; // whose turn it is
    private int onlineState; // whether i am online or not
    private int myPlayerNum;
    private String userName1 = ""; // always used - this player when online, 
                                   // player 1 when offline.
    private String userName2 = ""; // only used in local games for player 2.
    private String gameWinner = ""; // winner of a game.
    private String hostName = "";  // hostname of game server
    private int numP1Moves = 0;
    private int numP2Moves = 0;
    private GameBoard board;
    private GuiWindow gui;
    private boolean isLeverPulled = false;
    private boolean isStatSent = false; 

	private PlaySounds play = new PlaySounds();

    // constructor
    public ConnectN() {	
	setGameState(STARTUP);
	board = new GameBoard(7,6,4);
	gui = new GuiWindow(this);
	gui.start();
    }

    /**
     * getGameState will be used to give access to the current game state
     **/
    public int getGameState() {
	return( gameState );
    }    

    /**
     * setGameState will be used to set the current state of the game
     **/
    public void setGameState(int newState) {
	gameState = newState;
    }
    
    /**
     * getPlayerState will be used to give access to the current Player state
     * that is, it returns whose turn it is.
     **/
    public int getPlayerState() {
	return( playerState );
    }    

    /**
     * setPlayerState will be used to set the current state of the Player
     * that is, it sets whose turn it is.
     **/
    public void setPlayerState(int newState) {
	playerState = newState;
    }

    /**
     * getOnlineState will be used to give access to the current online state
     * that is, whether we're online or not.
     **/
    public int getOnlineState() {
	return (onlineState);
    }
    
    /** 
     * setOnLineState will be used to set the current online state
     * that is, it sets if we're online or not.
     **/
    public void setOnlineState(int newState) {
	onlineState = newState;
	return;
    }
    

    /**
     * isStatSend checks to see if this client has sent a stat line to
     * the server during this game.
     **/
    public boolean isStatSent() {
	return this.isStatSent;
    }

    public void setStatSent(boolean value) {
	this.isStatSent = value;
    }

    public boolean isLeverPulled() {
	return this.isLeverPulled;
    }

    public void setLeverPulled(boolean value) {
	this.isLeverPulled = value;
    }

    /**
     * setHost saves the host that the player is connected to during an online
     * state
     **/
    public void setHostName(String host) {
	this.hostName = host;
    }

    /**
     * getHost returns the hostname of the game server the player is connected
     * to
     **/
    public String getHostName() {
	return(this.hostName);	
    }

    /** 
     * getPlayerNum will be used to get whether this player is 1 or 2 
     **/
    public int getPlayerNum() {
	return myPlayerNum;
    }

    /**
     * setPlayerNum will be used to set whether this player is 1 or 2
     **/
    public void setPlayerNum(int pNum) {
	myPlayerNum = pNum;
    }

    /** 
     * getNumP1Moves() - gets the number of moves p1 has performed
     **/
    public int getNumP1Moves() {
	return numP1Moves;
    }    

    public int getNumP2Moves() {
	return numP2Moves;
    }

    public void setUserName1(String user) {
	this.userName1 = user;
    }

    public String getUserName1() {
	return this.userName1;
    }

    public void setUserName2(String user) {
	this.userName2 = user;
    }

    public String getUserName2() {
	return this.userName2;
    }

    public void setGameBoard(GameBoard newBoard) {
	this.board = newBoard;;
    }

    public GameBoard getGameBoard() {
	return this.board;
    }

    public GuiWindow getGui() {
	return this.gui;
    }
    
    public void setWinner(String winner) {
	this.gameWinner = winner;
    }

    public String getWinner() {
	return this.gameWinner;
    }

    // get the username of the current player, whether in a network game
    // or local game.
    public String getCurPlayer() {
	// if we're in an online game
	if(getOnlineState() == ONLINE) {
	    // if it is my turn
	    if(getPlayerState() == getPlayerNum())  {
		return(getUserName1());
	    }
	    
	    // otherwise it is the other player's turn 
	    else {
		return(getUserName2());
	    }	
	}

	// else we're in a local game
	else {
	    if(getPlayerState() == PLAYER_ONE_TURN) {
		return(getUserName1());
	    }
	    else if(getPlayerState() == PLAYER_TWO_TURN) {
		return(getUserName2());
	    }
	}
	// unreachable return
	return("");
    }

    /**
     * This method is used to reset the game states, chat, and gui displays
     * when a new game is selected.
     **/
    public void resetGame() {
	setGameState(OUTGAME);
	setPlayerNum(-1);
	setPlayerState(-1); 
	setOnlineState(OFFLINE);
	numP1Moves = 0;
	numP2Moves = 0;
	setUserName1("");
	setUserName2("");
	setWinner("");
	board.empty();
	setStatSent(false);
	setLeverPulled(false);
	gui.updateDisplayBoard();
    }

    public void handleDisconnect() {	    
	
	if(getOnlineState() == WAITING) {
	    setGameState(OUTGAME);
	    gui.updateDisplayBoard();
	}
	
	else if(getOnlineState() == ONLINE) {
	    if( (getGameState() != WINGAME) && (getGameState() != DRAWGAME) ){
		setWinner(userName1);
		setPlayerState(getPlayerNum());
		setGameState(WINGAME);
		gui.updateDisplayBoard();	    
	    }
	    
	}

	setOnlineState(OFFLINE);
	
    }
    
    public void handleReplayGame() {
		numP1Moves = 0;
		numP2Moves = 0;
		isStatSent = false;
		isLeverPulled = false;
		setPlayerState(PLAYER_ONE); // player 1's turn.
		setGameState(INGAME);
    }

    public void handleConnect(int playerNum) {
	if(playerNum == PLAYER_ONE) {	    
	    setPlayerNum(PLAYER_ONE);
	}

	else {
	    setPlayerNum(PLAYER_TWO);	    
	}

	setGameState(INGAME);
	setOnlineState(ONLINE);
	setPlayerState(PLAYER_ONE_TURN);
	gui.resetChatWindow();

    }

    public void newLocalStates() {
	
	setGameState(INGAME);
	setPlayerState(PLAYER_ONE_TURN);
	setOnlineState(OFFLINE);
	
	gui.updateDisplayBoard();
    }

    public void handleWinGame() {
	setGameState(WINGAME);
	String winner = getCurPlayer();
	setWinner(winner);
	gui.updateDisplayBoard();
	if(getOnlineState() == ONLINE){
		if(getWinner() == getUserName1()){
			play.playAudio("winner.aif");
		}else{
			play.playAudio("loser.aif");
		}
	}else{
		play.playAudio("winner.aif");
	}
    }

    
    public void handleDrawGame() {
	setGameState(DRAWGAME);
	setLeverPulled(false);
	gui.updateDisplayBoard();
    }

     /**
     * updateGame() - this method updates the GameBoard, the game state,
     * and the game display. Assumes the column number is valid for the
     * current board size.
     **/
    public boolean updateGame(int column) {
	boolean set = false;
	char player;

	// set whose move it is.
	if( getPlayerState() == PLAYER_ONE_TURN ){ player = 'r'; }
	else { player = 'b'; }
	
	set = this.board.execute(player, column);
	if( set ) {

	    // increment the number of moves they player has performed
	    if( getPlayerState() == PLAYER_ONE_TURN ){ numP1Moves++; }
	    else { numP2Moves++; }
	    
	    // test if the player won
	    if( board.isWin(player) ) {
		handleWinGame();
	    }
	    
	    // else test if there's a draw.
	    else if( board.isDraw() ) {
		handleDrawGame();
	    }
	    
	    // if its not a win, and not a draw, make it the other guy's turn.
	    else if ( getPlayerState() == PLAYER_ONE_TURN ) {
		setPlayerState(PLAYER_TWO_TURN); 
	    }
	    else { setPlayerState(PLAYER_ONE_TURN); }

	}
	
	// piece was not set, display an error.
	else { gui.updateChatWindow("Piece Placement not allowed: " +
				    "that column is full.");
	play.playAudio("uh oh.aif");
	}
	gui.updateChatWindow("Piece placed in column " + (column+1));
	gui.updateDisplayBoard();
	
	if(set)
		play.playAudio("chip.aif");

	//return whether the move was successful
	return(set);
    }

    // loads a local game from file. if error, prints to chat window.
    public boolean loadLocalGame(String filename) { 
	GameBoard newBoard = null;
	try {
	  BufferedReader reader = new BufferedReader(new FileReader(filename));
	  String line;
	  
	  line = reader.readLine();
	  if(line.equals("7x6")) {
	      newBoard = new GameBoard(7,6,4);
	  }
	  else if(line.equals("6x8")) {
	      newBoard = new GameBoard(6,8,4);
	  }
	  else if(line.equals("11x12")) {
	      newBoard = new GameBoard(11,12,5);
	  }	  
	  else { 
	      gui.updateChatWindow("An Error occured loading your file from " 
				   + filename);
	      return false;
	  }

	  this.board = newBoard; // set the new board.
	  
	  userName1 = reader.readLine().trim();
	  userName2 = reader.readLine().trim();
	  playerState = Integer.parseInt(reader.readLine().trim());

	  for (int row=0; row<board.getHeight(); row++) {
	      line = reader.readLine().trim();
	      for (int col=0; col<board.getWidth(); col++) {
		  if (line.charAt(col) == 'r') 
		      board.setPosition(col, row, 'r');
		  else if (line.charAt(col) == 'b')
		      board.setPosition(col, row, 'b');
		  else
		      board.setPosition(col, row, 'e');
	      }
	  }
	  
	  gameState = INGAME;
	  onlineState = OFFLINE;

	  gui.updateDisplayBoard();

	  return true;
	}
	catch (IOException e) { }
	catch (NumberFormatException e) {}
	gui.updateChatWindow("An Error occured loading your file from " 
			     + filename);
	return false;
    }

    /**
     * saveLocalGame(filename) - save a game to filename.
     */
    public boolean saveLocalGame(String filename) {
	try {
	    PrintWriter writer = new PrintWriter(new FileWriter(filename));
	    writer.print("" + board.getWidth() + "x" + board.getHeight() + 
			 "\n" + userName1 + "\n" + userName2 +"\n" +
			 playerState + "\n");
	    for (int row=0; row<board.getHeight(); row++) {
		for (int col=0; col<board.getWidth(); col++) {
		    if (board.getPosition(col, row) == 'r')
			writer.print("r");
		    else if (board.getPosition(col, row) == 'b')
			writer.print("b");
		    else
			writer.print("e");
		}
		writer.print("\n");
	    }
	    writer.close();
	    return true;
	} 
	catch (IOException e) { }
	catch (NumberFormatException e) {}
	gui.updateChatWindow("An error occured saving your game to " +
			     filename);
	
	return false;
    }    

    public static void main (String args[]) throws java.io.IOException {
	ConnectN game = new ConnectN();	
    } // end main method
    
} // end ConnectN class.

