/******************************************************************************
 * playerCommunicator.java - a thread that handles communication to and from 
 * another player in a connect N game.
 * 
 *
 *
 *
 *****************************************************************************/
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.util.*;

class PlayerCommunicator extends Thread implements ActionListener, WindowListener {
    private ConnectN game;
    private GuiWindow gui;
    private Socket playerSocket = null;
    private BufferedReader playerInput;
    private PrintWriter playerOutput;
    private Socket serverSocket = null;
    private JFrame chooseBoardFrame;
    private boolean isBoardChosen = false;
    private String selectedSize = "";

    PlayerCommunicator(ConnectN gm, GuiWindow gui, Socket serverSock) {
	this.game = gm;
	this.gui = gui;
	this.serverSocket = serverSock;
    }

    public void run() {
	String gameInfo = "";
	String boardSize = "";
	String line = "";
	String firstToken = "";
	GameBoard board = null;
	StringTokenizer st;

	try {
	    // open input/output streams to server
	    BufferedReader serverInput = new BufferedReader(
				   new InputStreamReader(
				   serverSocket.getInputStream()));
	    PrintWriter serverOutput = new PrintWriter(
                                 serverSocket.getOutputStream());

	    // begin authentication
	    serverOutput.println("connect");
	    serverOutput.flush();

	    // get a line from the negotiating server.
	    line = serverInput.readLine();

	    // if line is null, something terrible happend. 
	    if(line == null) {
		disconnect();
		return;
	    }
	    // if the server says we are the server, then we will act as server
	    // for this game. (player 1)
	    if(line.equals("server")) {

		// call playerOneSetup, which returns the socket we're 
		// listening on for the client's connection.
		ServerSocket mySocket = playerOneSetup(serverInput,
						       serverOutput);
		if(mySocket == null) {
		    System.exit(1);
		}

		// close the negotiating server connection.
		serverSocket.close();
		serverSocket = null;

		// wait for player two's connection, then set up reader streams
		this.playerSocket = mySocket.accept();
		this.playerInput = new BufferedReader(new InputStreamReader(
					  this.playerSocket.getInputStream()));
		this.playerOutput = new PrintWriter(
					  this.playerSocket.getOutputStream());

		chooseNetBoardSize();	

		// block until the board is chosen in the gui.
		while(isBoardChosen == false) { }

		// put the board size in a string. 
		board = game.getGameBoard();
		boardSize = "" + board.getWidth() + "x" + board.getHeight();

		this.sendLine(boardSize);
		// handle connected states
 		game.handleConnect(game.PLAYER_ONE);
		
		// send our username
		this.sendLine("username " + game.getUserName1());
		this.listen();
	    }

	    // in this case we were told by the negotiating server that we
	    // are the client (player 2).
	    else if(line.equals("client")) {
		
		// call playerTwoSetup, which returns a string holding the
		// connection info for player 1 and the chosen board size.
		gameInfo = playerTwoSetup(serverInput, serverOutput);
		if(gameInfo == null) {

		    System.exit(1);
		}		    

		// parse the gameInfo for player 1's ip, port, and chosen board
		// size. 
		st = new StringTokenizer(gameInfo);
		String p1IP = st.nextToken();
		String p1PortString = st.nextToken();
		int p1Port = Integer.parseInt(p1PortString);		
	
		// close the negotiating server connection.
		serverSocket.close();
		serverSocket = null;

		// here we need to instantiate a game board of size gameBoard.
		
		// open socket to player 1. 
		this.playerSocket = new Socket(p1IP, p1Port);
		this.playerInput = new BufferedReader(new InputStreamReader(
					  this.playerSocket.getInputStream()));
		this.playerOutput = new PrintWriter(
					  this.playerSocket.getOutputStream());
		
		String game_board = this.playerInput.readLine();
		if( (game_board == null) || (game_board.equals("disconnect")) ){ 
		    disconnect(); 
		    return;
		}
		st = new StringTokenizer(game_board);
		boardSize = st.nextToken();
		
		GuiWindow gui = game.getGui();
		board = game.getGameBoard();
		if(boardSize.equals("7x6")) {
		    game.setGameBoard(new GameBoard(7,6,4));
		    gui.updateDisplayBoard();
		}
		else if(boardSize.equals("6x8")) {
		    game.setGameBoard(new GameBoard(6,8,4));
		    gui.updateDisplayBoard();
		}
		else if(boardSize.equals("11x12")) {
		    game.setGameBoard(new GameBoard(11,12,5));
		    gui.updateDisplayBoard();
		}

		game.handleConnect(game.PLAYER_TWO);

		// send our username
		this.sendLine("username " + game.getUserName1());
		this.listen();
	    }
	}
 
	catch(IOException e) {
	    gui.updateChatWindow("There was an error on the game server. Please try" +
				 " again");
	    disconnect();
	    return;
	}

    } // end run method
    
    public ServerSocket playerOneSetup(BufferedReader serverInput, 
					      PrintWriter serverOutput) 
	                                      throws IOException {
	int myPort = 5555;
	int MAXPORTS = 65536;         // upper bound on valid ports
	boolean isServerUp = false;
	String line = "";
	String gameInfo = "";         // holds info player 2 needs to connect.
	String boardSize = "7x6";     // board size, for test purposes only.
	ServerSocket mySocket = null; // socket I listen on to for player 2.

	
	// by default we try to open a server on port 5555. if this
	// doesn't work, increment and try again.
	while(isServerUp == false) {
	    try {
		mySocket = new ServerSocket(myPort);
		isServerUp = true;
	    }
	    
	    catch(IOException e) {
		myPort++;

		// if we try so many ports that we reach the limit, print an
		// error and return null.
		if(myPort > MAXPORTS) {
		    mySocket = null;
		    return(mySocket);
		}
	    }	   	    
	}	

	// create a string that contains the port my server is opened on,
	// and the board size. this string will be transmitted to the
	// negotiating server for client 2 to use to connect to me.
	// this string is in the following format "<port> <board size>"	
	gameInfo = "" + myPort;
	
	// transmit the string to negotiating server.
	serverOutput.println(gameInfo);
	serverOutput.flush();

	// server should acknowledge our transmission. if not, return null.
	line = serverInput.readLine();
	if(!line.equals("ok")) {
	    mySocket.close();
	    mySocket = null;
	    return(mySocket);
	}
	
	// we received acknowledgement, and return our socket that
	// was setup as a server.
	return(mySocket);

    } // end playerOneSetup method

    public static String playerTwoSetup(BufferedReader serverInput, 
					PrintWriter serverOutput) 
	                                throws IOException {
	String boardSize = "";
	String gameInfo = "";
	StringTokenizer st = null;

	// accept our role as client.
	serverOutput.println("ok");
	serverOutput.flush();
	
	// Server will respond with necessary info to connect to player 1
	// and player 1's chosen board size. This info should be
	// in the following format: <IP> <port> <board size>
       gameInfo = serverInput.readLine();
	
	// perform sanity check on gameInfo, checking to make sure
	// there are 3 tokens on the line. if not, complain and return null.
	st = new StringTokenizer(gameInfo);
	if(st.countTokens() != 2) {
	    gameInfo = null;
	    return(gameInfo);
	}
	
	// line is kosher. return it.
	return(gameInfo);		

    } // end of playerTwoSetup method	

    // sendLine(): 
    // sends a line to the other player, whether it be command, notice,  
    // or chat line.
    public void sendLine(String lineToSend) {
	this.playerOutput.println(lineToSend);
	this.playerOutput.flush();
    }

    public void listen() {
	String line = "";
	String stToken = "";
	StringTokenizer st;
	// listen forever. actually, when the other player disconnects, it will
	// throw an exception below. 
	while(true) {
	    try {
		// get the line and tokenize it.
		line = this.playerInput.readLine();
		if(line == null) {
		    break;
		}
		st = new StringTokenizer(line);
		stToken = st.nextToken();
		// test first token for various possibilities.
		if(stToken.equals("move")) {
		    // place piece on board.
		    int columnNumber = Integer.parseInt(st.nextToken().trim());
		    game.updateGame(columnNumber);
		} 

		else if(stToken.equals("chat")) {
		    String lineToSend = "";
		    // strip the "chat" command out of the line.
		    while(st.hasMoreTokens()) {
			lineToSend = lineToSend + " " + st.nextToken();
		    }
		    gui.updateChatWindow(lineToSend.trim());  
		}

		// if the other client sent their player's username
		else if(stToken.equals("username")) {
		    String otherPlayerName = "";
		     while(st.hasMoreTokens()) {
		      otherPlayerName = otherPlayerName + " " + st.nextToken();
		     }

		     // in a network game, playerName1 is always the username
		     // of this player, playerName2 is always the username
		     // of the other player.
		     game.setUserName2(otherPlayerName.trim());
		     
		     // update the gui based in the new name.
		     GuiWindow gui = game.getGui();
		     gui.updateDisplayBoard();
		}
		else if(stToken.equals("you")) {
		    // see if we won
		}
		 
		else if(stToken.equals("disconnect")) {
		    // disconnect. 		
		    gui.updateChatWindow("Your opponnent disconnected.");
		    disconnect();
		}
		
		else if(stToken.equals("disconnectreset")) {
		    // disconnect. 
			this.game.resetGame();
		    gui.updateChatWindow("Your opponnent disconnected.");
		    disconnect();
		}
		
		else if(stToken.equals("empty")) {
		    // winner pulled the arm empty the board. 		    
		    this.game.getGameBoard().empty();
		    this.gui.resetChatWindow();
		    this.gui.updateDisplayBoard();
		}

		else if(stToken.equals("replay")) {
		    // winner wants a replay. 		    
			this.game.handleReplayGame();
		}

		else { }
	    }

	    // catch IOException. occurs when there's a disconnect or error
	    // getting a line.
	    catch(IOException e) {
		disconnect();
		return;
	    }	    
	}
    } // end of listen() method

    private void chooseNetBoardSize() {
	JFrame.setDefaultLookAndFeelDecorated(true);
	chooseBoardFrame = new JFrame("Board Size Selection");
	chooseBoardFrame.setBackground(ConnectN.GUICOLOR);
	chooseBoardFrame.setLocationRelativeTo(gui.getGuiFrame());
	JButton ok = new JButton("OK");
	ok.setBackground(ConnectN.GUICOLOR);
	ok.setActionCommand("okNetBoardSize");
	ok.addActionListener(this);

	JPanel thePanel = new JPanel(new BorderLayout());
	thePanel.setBackground(ConnectN.GUICOLOR);
	JPanel radioPanel = new JPanel(new GridLayout(0,1));
	radioPanel.setBackground(ConnectN.GUICOLOR);
	JLabel radioLabel = new JLabel("Select a Board Size: ");
	radioLabel.setBackground(ConnectN.GUICOLOR);
	JPanel buttonPanel = new JPanel(new GridLayout(0,1));
	buttonPanel.setBackground(ConnectN.GUICOLOR);
	buttonPanel.add(ok);

	// radio buttons
	JRadioButton sevBySixButton = new JRadioButton("7x6");
	sevBySixButton.setBackground(ConnectN.GUICOLOR);
	sevBySixButton.setActionCommand("sevBySixButton");
	sevBySixButton.addActionListener(this);

	JRadioButton sixByEitButton = new JRadioButton("6x8"); 
	sixByEitButton.setBackground(ConnectN.GUICOLOR);
	sixByEitButton.setActionCommand("sixByEitButton");
	sixByEitButton.addActionListener(this);

	JRadioButton elvByTwvButton = new JRadioButton("11x12");
	elvByTwvButton.setBackground(ConnectN.GUICOLOR);
	elvByTwvButton.setActionCommand("elvByTwvButton");
	elvByTwvButton.addActionListener(this);
	
	// do drop down menu for board sizes.
	ButtonGroup group = new ButtonGroup();
	group.add(sevBySixButton);
	group.add(sixByEitButton);
	group.add(elvByTwvButton);
	
	radioPanel.add(radioLabel);
	radioPanel.add(sevBySixButton);
	radioPanel.add(sixByEitButton);
	radioPanel.add(elvByTwvButton);
	radioPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	thePanel.add(radioPanel, BorderLayout.CENTER);
	thePanel.add(buttonPanel, BorderLayout.SOUTH);
	chooseBoardFrame.addWindowListener(this);
	chooseBoardFrame.getContentPane().add(thePanel, BorderLayout.CENTER);
	chooseBoardFrame.pack();
	chooseBoardFrame.show();
    }

    public void actionPerformed(ActionEvent e) {
	GameBoard board = game.getGameBoard();

	// handle when the user is player 1 and chooses a board size for the network
	// game. 
	if(e.getActionCommand().equals("okNetBoardSize")) {
	    

	    // if 7x6 was selected
	    if(selectedSize.equals("sevBySixButton")) {
		board = new GameBoard(7, 6, 4);
		game.setGameBoard(board);
		gui.updateDisplayBoard();
		isBoardChosen = true;
		chooseBoardFrame.setVisible(false);
		chooseBoardFrame.dispose();
	    }
	    // if 6x8 was selected
	    else if (selectedSize.equals("sixByEitButton")) {
		board = new GameBoard(6, 8, 4);
		game.setGameBoard(board);
		gui.updateDisplayBoard();
		isBoardChosen = true;
		chooseBoardFrame.setVisible(false);
		chooseBoardFrame.dispose();
	    }
	    // if 11x12 was selected
	    else if (selectedSize.equals("elvByTwvButton")) {
		board = new GameBoard(11, 12, 5);
		game.setGameBoard(board);
		gui.updateDisplayBoard();
		isBoardChosen = true;
		chooseBoardFrame.setVisible(false);
		chooseBoardFrame.dispose();
	    }

	    // if no radio button was selected
	    else {
		gui.updateChatWindow("Game error: You didn't select a board size.");
	    }
	    

	    
	}

	// handle radio button selection of board size. 
	else if(e.getActionCommand().equals("sevBySixButton") ||
		e.getActionCommand().equals("sixByEitButton") ||
		e.getActionCommand().equals("elvByTwvButton") ) {

	    // save the chosen radio button board size.
	    selectedSize = e.getActionCommand();
	}
	
	// handle cancel gets pressed on any dialogue box.
	else if(e.getActionCommand().equals("cancelButton") ) {
	    chooseBoardFrame.setVisible(false);
	    chooseBoardFrame.dispose();
	}

    } // end ActionPerformed method

    /**
	 * Window Listener
	 **/
	public void windowClosing(WindowEvent e)
	{ 
	    sendLine("disconnect");
	    disconnect();
	}
    public void windowClosed(WindowEvent e) {
        //This will only be seen on standard output.

    }

    public void windowOpened(WindowEvent e) {
        //System.out.println("WindowListener method called: windowOpened.");
    }

    public void windowIconified(WindowEvent e) {
        //System.out.println("WindowListener method called: windowIconified.");
    }

    public void windowDeiconified(WindowEvent e) {
        //System.out.println("WindowListener method called: windowDeiconified.");
    }

    public void windowActivated(WindowEvent e) {
        //System.out.println("WindowListener method called: windowActivated.");
    }

    public void windowDeactivated(WindowEvent e) {
		// System.out.println("WindowListener method called: windowDeactivated.");
    }

    public void windowGainedFocus(WindowEvent e) {
        //System.out.println("WindowFocusListener method called: windowGainedFocus.");
    }

    public void windowLostFocus(WindowEvent e) {
		// System.out.println("WindowFocusListener method called: windowLostFocus.");
    }

    public void windowStateChanged(WindowEvent e) {
		// System.out.println(
        //  "WindowStateListener method called: windowStateChanged." + e);
	}
    public void disconnect() {
	try {
	    if(playerSocket != null) {
		playerSocket.close();
		playerSocket = null;
	    }
	    if(serverSocket != null) {
		serverSocket.close();
		serverSocket = null;
	    }

	    game.handleDisconnect();	    
	}

	catch(IOException e) {
	}
    }
} // end playerCommunicator class
