/******************************************************************************
 *
 *
 *****************************************************************************/
import java.io.*;
import java.net.*;
import java.util.*;

class ClientNegotiator extends Thread {
    private Socket clientOneSock;
    private Socket clientTwoSock;

    // Constructor
    ClientNegotiator(Socket client1, Socket client2) {
	this.clientOneSock = client1;
	this.clientTwoSock = client2;
    }
    
    // Thread begins
    public void run() { 
	String line = "";
	String curToken = "";
	boolean isNegotiated = false;

	try {	    
	    // open an input stream and output stream with each client's
	    // socket.
	    BufferedReader oneInput = new BufferedReader(
				   new InputStreamReader(
				   clientOneSock.getInputStream()));
	    PrintWriter oneOutput = new PrintWriter(
                                 clientOneSock.getOutputStream());
	    BufferedReader twoInput = new BufferedReader(
				   new InputStreamReader(
				   clientTwoSock.getInputStream()));
	    PrintWriter twoOutput = new PrintWriter(
                                 clientTwoSock.getOutputStream());

	    isNegotiated = Negotiate(oneInput, oneOutput, twoInput, 
				     twoOutput);
	    if(isNegotiated == false) {
		System.out.println("ClientNegotiator Failed in client " +
				   "negotiation.");
	    }
	    
	    else {
		System.out.println("clientNegotiator Successfully Negotiated" +
					   " with clients.");		
	    }

	    // close client sockets
	    clientOneSock.close();
	    clientTwoSock.close();	    
	    
	    return;
	}

	catch (IOException e) {	    
	    System.out.println("clientNegotiator Failed in client " +
			       "Negotiation.");
	    System.out.println( e );
	}

    } // end run() method

    public boolean Negotiate(BufferedReader oneInput, PrintWriter oneOutput, 
			     BufferedReader twoInput, PrintWriter twoOutput) {
	String line = "";
	String gameInfo = "";
	StringTokenizer st = null;
	boolean isSuccess = false;

	try {
	    // get a line from player 1. it should hold "connect". If it 
	    // doesn't, complain and return.
	    line = oneInput.readLine();
		if(line == null) {
			disconnect();
			return false;
		}	    
		line = line.trim();

	    if(!line.equals("connect")) {
		System.out.println("Protocol violation: first line received " +
				   "from client 1 was not 'connect'");
		isSuccess = false;
		return(isSuccess);
	    }
	    
	    // "connect" was received, tell client 1 that it is the server.
	    // client 1 should then reply with its port and game board 
	    // size. If it doesn't reply in the correct format, complain 
	    // and return. 
	    oneOutput.println("server");
	    oneOutput.flush();	    
	    gameInfo = oneInput.readLine();
		if(gameInfo == null){
			disconnect();
			return false;
		}
		gameInfo = gameInfo.trim();		st = new StringTokenizer(gameInfo);
		if(st.countTokens() != 1) {
		System.out.println("Protocol violation: Wrong number of " +
				   "line tokens.");
		isSuccess = false;
		return(isSuccess);
	    }
	    
	    // the line had 2 tokens, good! tell the server we got the info.
	    oneOutput.println("ok");
	    oneOutput.flush();
	    // player 1 should disconnect at this point, but thats not the
	    // problem of the server, so we don't handle it.
	    
	    // now figure out client 1's ip so we can give it to player 2.
	    String playerOneIP = clientOneSock.getInetAddress().toString();

	    // for some reason, getInetAddress() returns the ip with a 
	    // slash at the beginning of the string. we must remove it.
	    playerOneIP = playerOneIP.substring(1);

	    // append gameInfo to client 1's IP string so that it holds
	    // the following info: <IP address> <port>. This is
	    // all the info that gets sent to client 2.
	    gameInfo = playerOneIP + " " + gameInfo;

	    // get a line from client 2. if it doesnt hold "connect", complain
	    // and return.
	    line = twoInput.readLine();
		if(line == null) {
			disconnect();
			return false;
		}
		line = line.trim();
	    if(!line.equals("connect")) {
		System.out.println("Protocol violation: " +
				   "first line received not 'connect'");
		isSuccess = false;
		return(isSuccess);
	    }

	    // tell client 2 that it is the client in the game. if it doesn't
	    // acknowledge correctly, complain and return.
	    twoOutput.println("client");
	    twoOutput.flush();
	    
	    line = twoInput.readLine()   ;
		if(line == null) {
			disconnect();
			return false;
		}
		line = line.trim();
	    if(!line.equals("ok")) {
		System.out.println("Protocol violation: " +
				   "client 2 did not accept its role as " +
				   "client!");
		isSuccess = false;
		return(isSuccess);
	    }

	    // since client 2 accepts its role as client, send it gameInfo.
	    twoOutput.println(gameInfo);
	    twoOutput.flush();
	    
	    // everything is set up correctly.
	    isSuccess = true;
	    return(isSuccess);
	}

	catch(IOException e) {
	    isSuccess = false;
	    return(isSuccess);
	}

    } // end method Negotiate

	public void disconnect() {
		if(clientOneSock != null){
			try{
			clientOneSock.close();
			}
			catch (IOException e) {	    
				System.out.println( e );
			}
		}
		if(clientTwoSock != null){
			try{
			clientTwoSock.close();
			}
			catch (IOException e) {	    
				System.out.println( e );
			}
		}
	}

	
} // end class clientNegotiator
