/******************************************************************************
 * CREATED: 05/02/2004
 * Stats.java - This is a thread-safe stats object. The only real information
 * is in the 'entries' vector, which is a vector of strings. Each entry in
I* the vector is a stats entry in the format: 
 * <timestamp> <username> <num connected> <num moves>
 *
 * This class is synchronized. it does so on the inUse boolean variable.
 * when one of the methods is called, it checks to see if inUse is true. if so,
I* the method wait()'s until it receives a notify(). This is to avoid two
 * clients connecting to the clientListener and borking each other's stats 
 * updates. 
 *
 * the stats constructor should be instantiated in clientListener BEFORE 
 * it is passed to the statsUpdater thread for stats update/retrieval. 
 * This is because the constructor opens the file stats.txt and loads the 
 * entries into its 'entries' vector.
 *****************************************************************************/
import java.lang.*;
import java.io.*;
import java.util.*;

class Stats {
    private Vector entries;        // vector of statistical entries
    private boolean inUse = false;  

    // constructor stats() instantiates empty Vector and then loads the vector
    // with the contents of stats.txt
    public Stats() throws IOException { 
	String line = "";
	try {
	    BufferedReader infile = 
		new BufferedReader(new FileReader("stats.txt"));

	    this.entries = new Vector(100,100);
  
	    for(line = infile.readLine(); line != null; 
		line = infile.readLine()) {
		
		this.entries.add(line);	    
	    }	
	    
	    infile.close();
	}

	catch(IOException e) {
	    System.out.println("could not open stats.txt. starting with" +
			       "empty stats database");
	    this.entries = new Vector(100,100);
	    return;
	}

	
    }

    // toString returns a string representation of all stats entries.
    public synchronized String toString() { 
	String curEntries = "";
	while(inUse) {
	    try {
		wait();
	    }
	    catch(InterruptedException e) {
		
	    }
	}
	this.inUse = true;
	curEntries = entries.toString();
	this.inUse = false;
	notify();
	return curEntries; 
    }

    // accessor methods: return each vector of stats information.    
    public synchronized int getNumEntries() { 
	int numEntries;
	while(this.inUse) {
	    try {
		wait();
	    }
	    catch(InterruptedException e) {
		
	    }
	}
	this.inUse = true;
	numEntries = this.entries.size();
	this.inUse = false;
	notify();
	return numEntries;
    }
    public synchronized Object getEntries() { 
	Vector curEntries;
	while(this.inUse) {
	    try {
		wait();
	    }
	    catch(InterruptedException e) {
		
	    }
	}
	this.inUse = true;
	curEntries = this.entries;
	this.inUse = false;
	notify();
	return curEntries.clone(); 
    }

    // insert method - insert a new stats entry into the DB
    public synchronized void statInsert(String newEntry) throws IOException { 
	int i;
	
	while(this.inUse) {
	    try {
		wait();
	    }
	    catch(InterruptedException e) {
		
	    }
	}

	this.inUse = true;
	this.entries.add(0, newEntry);

	PrintWriter outfile = new PrintWriter(new FileWriter("stats.txt"));

	for(i = 0; i < entries.size(); i++) {
	    outfile.println(entries.get(i));
	}

	outfile.close();

	this.inUse = false;
	notify();
    }

} // end class Stats

