/**
 * DisplayBoard is the game board as it is displayed
 * to in the GUI.
 *
 * Pardon Our French
 * date created: 5/7/2004
 * last revision: 5/23/2004
 **/
import java.awt.*;
 
public class DisplayBoard
{

    // Class variables.
    
    // Instance variables.
    private int columns;
    private int rows;
	private int columnWidth;
	private int rowHeight;


    /**
     * Constructor for class DisplayBoard
     */
    public DisplayBoard(int columns, int rows)
    {
	this.columns = columns;
	this.rows = rows;       
    }

	/**
	 * Returns the column that relates to the X value that
	 * is entered.
	 **/
	public int getColumn(int mouseY, int startX, int endX)
	{
		return( (mouseY) % (endX - startX) );
	}

    /**
     * set columns and rows to new sizes
     **/
    public void setDisplayBoardSize(int cols, int rows) {
	this.columns = cols;
	this.rows = rows;
    }

    /**
     * Draws a grid on the screen.
     */
    public void drawGrid(Graphics gfx, Color gridColor, int startX, int startY
			 , int endX, int endY) {
   

		//Set the width
		int width = endX-startX;

		//Set the height
		int height = endY-startY;

		//Set the Column Width to
		// endX-startX / this.columns
		columnWidth = width / columns;
		
		//Set the Row Height to
		// endY-startY / this. rows
		rowHeight = height / rows;

		// Set the board color.
		gfx.setColor(gridColor);

		// Draw the boardbackground
		gfx.fillRect(startX, startY, this.columns*columnWidth,
					 this.rows*rowHeight);

		// Set the line color.
        gfx.setColor(Color.gray);

        // Draw the horizontal lines.
        int lineCounter = 1;
        int x = startX;
        int y = startY;
        while (lineCounter <= this.rows + 1) {
            gfx.drawLine(x, y, x + columnWidth * this.columns, y);
            y = y + rowHeight;
            lineCounter++;
        }
	
        // Draw the vertical lines.
        lineCounter = 1;
        x = startX;
        y = startY;
		while (lineCounter <= this.columns + 1) {
			gfx.drawLine(x, y, x, y + rowHeight * this.rows);
			x = x + columnWidth;
            lineCounter++;
		}

		// Draw the holes
		DisplayPiece currentPiece;
		for(int rise = 0; rise < this.rows; rise++) {	
			for(int count = 0; count < this.columns; count++) {
				// Make this display a gray piece at each spot
				currentPiece = new DisplayPiece();
				currentPiece.setColor(Color.gray);
				currentPiece.setSize(findHoleSize());
				currentPiece.setLocation(findCenterX(count),
										 findCenterY(rise));
				currentPiece.drawPiece(gfx);
				currentPiece.setColor(ConnectN.GUICOLOR);
				currentPiece.setSize(findHoleSize()-2);
				currentPiece.setLocation(findCenterX(count)+1,
										 findCenterY(rise)+1);
				currentPiece.drawPiece(gfx);

			}
		}

    }

	/**
	 * findSize returns and int that will be the minimum of the column
	 * width and the row height
	 **/
	public int findHoleSize()
	{
		if( columnWidth < rowHeight )
			return( columnWidth - 6 );
		else
			return( rowHeight - 6 );
	}


	/**
	 * findCenterX returns an int for where the center of the column
	 * is for a given column
	 **/
	public int findCenterX(int column)
	{
		int offset = (columnWidth - findHoleSize()) / 2;
		int center = column * columnWidth + offset;
		return(center);
	}

	/**
	 * findCenterY returns an int for where the center of the row
	 * is for a given row
	 **/
	public int findCenterY(int row)
	{
		int offset = (rowHeight - findHoleSize()) / 2;
		int center = row * rowHeight + offset;
		return(center);
	}

}
