/**
 * DisplayPanel is the panel where the board is displayed. It draws a yellow
 * board over a grey background. It also implements the mouselistener so
 * it can be used to tell where to place the next piece.
 **/
import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.Vector;
import java.awt.image.*;


public class DisplayPanel extends JPanel
{
    private ConnectN game;
    private EventHandler listener;
    private static final int GRID_X = 0;
    private static final int GRID_Y = 0;
    private static final int DEFAULT_COLUMN = 7;
    private static final int DEFAULT_ROW = 6;
    private static final int PANEL_WIDTH = 500;
    private static final int PANEL_HEIGHT = 500;
	
    private DisplayBoard disBoard;
	private TexturePaint imagePaint;
	BufferedImage blackImage;
	BufferedImage redImage;
	BufferedImage blackImage2;
	BufferedImage redImage2;
	private String redImageFile = "redpiece.jpg";
	private String blackImageFile = "blackpiece.jpg";
	private String redImageFile2 = "redpiece2.jpg";
	private String blackImageFile2 = "blackpiece2.jpg";
	
	public DisplayPanel(ConnectN gm, EventHandler eventlistener) {
		this.game = gm;
		GameBoard board = game.getGameBoard();
		this.listener = eventlistener;
		this.disBoard = new DisplayBoard(board.getWidth(), board.getHeight());
		setBackground(ConnectN.GUICOLOR);
		setPreferredSize(new Dimension(PANEL_WIDTH, PANEL_HEIGHT));
		this.addMouseListener(listener);
		blackImage = ImageUtilities.getBufferedImage(blackImageFile, this);
		redImage = ImageUtilities.getBufferedImage(redImageFile, this);
		blackImage2 = ImageUtilities.getBufferedImage(blackImageFile2, this);
		redImage2 = ImageUtilities.getBufferedImage(redImageFile2, this);
	}
	
	public void setDisplayBoardSize(int cols, int rows) {
		this.disBoard.setDisplayBoardSize(cols, rows);
	}

	/** 
	 * Paints the inside of the panel. It draws the board.
	 **/
	public void paintComponent(Graphics g) {
		GameBoard board = game.getGameBoard();
		super.paintComponent(g);
		Graphics2D gfx = (Graphics2D)g;
		
		// Get the size of the client area.
		int width = super.getWidth() - GRID_X;
		int height = super.getHeight() - GRID_Y;
		
		this.disBoard.drawGrid(gfx, Color.yellow, GRID_X, GRID_Y,
							   width, height);
		
		// This section reads the board and places pieces on the gui
		// This makes a new image for use on the pieces



		int boardColumn = board.getWidth();
		int boardRow = board.getHeight();
		DisplayPiece currentPiece;
		char[][] currentBoard = board.getBoard();
		for(int rise = 0; rise < boardRow; rise++) {	
			for(int count = 0; count < boardColumn; count++) {
				// Make this display a red piece when an r is
				// read and a black piece when a b is read
				if( currentBoard[count][rise] == 'r'){
					currentPiece = new DisplayPiece();
					if( ( (count*rise) % 2 ) == 1 )
					imagePaint = new TexturePaint(redImage,
									new Rectangle(
									  disBoard.findCenterX(count),
									  disBoard.findCenterY(rise),
									  disBoard.findHoleSize(),
									  disBoard.findHoleSize()));
					else
					imagePaint = new TexturePaint(redImage2,
									new Rectangle(
									  disBoard.findCenterX(count),
									  disBoard.findCenterY(rise),
									  disBoard.findHoleSize(),
									  disBoard.findHoleSize()));		
					currentPiece.setTexture(imagePaint);
					currentPiece.setColor(Color.red);
					currentPiece.setSize(disBoard.findHoleSize());
					currentPiece.setLocation(disBoard.findCenterX(count),
											 disBoard.findCenterY(rise));
					currentPiece.drawPiece(gfx);
				}
				else{
					if( currentBoard[count][rise] == 'b'){
						currentPiece = new DisplayPiece();
						if( ( (rise*count) % 2 ) == 1 )
						imagePaint = new TexturePaint(blackImage,
								   	  new Rectangle(
									  disBoard.findCenterX(count),
									  disBoard.findCenterY(rise),
									  disBoard.findHoleSize(),
									  disBoard.findHoleSize()));
						else
						imagePaint = new TexturePaint(blackImage2,
								   	  new Rectangle(
									  disBoard.findCenterX(count),
									  disBoard.findCenterY(rise),
									  disBoard.findHoleSize(),
									  disBoard.findHoleSize()));			
						currentPiece.setTexture(imagePaint);
						currentPiece.setColor(Color.black);
						currentPiece.setSize(disBoard.findHoleSize());
						currentPiece.setLocation(disBoard.findCenterX(count),
												 disBoard.findCenterY(rise));
						currentPiece.drawPiece(gfx);
					}
					else{
						//Neither red nor black so must be empty
					}
				}
				
				//System.out.print(currentBoard[count][rise]);	
			}
			//	System.out.println();
		}
	}
}
