/**
 * DisplayPiece is the pieces as they are displayed
 * in the GUI.
 *
 * Pardon Our French
 * date created: 5/7/2004
 * last revision: 5/7/2004
 **/

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.geom.*;
 
public class DisplayPiece
{

    // Class variables.
    private static final int NEW_PIECE = 0;
    private static final int SET_PIECE = 1;

    // Instance Variables.
    private int x, y;
	private int size;
    private Color pieceColor;
    private int newOrSet;
	private TexturePaint imagePaint;
	private Ellipse2D imageOval;

    /**
     * Constructor for class DisplayPiece
     */
    public DisplayPiece()
    {
	newOrSet = NEW_PIECE;
    }

    /**
     * moveDisplayPiece is used to move a piece on the display. 
	 * When it is called
     * it creates a new piece for the board. It handles the animation of
     * the piece falling into its correct position.
     */
    public void movePiece(Graphics gfx, int x, int startY,
			 int endY)
    {
	// set color of piece
	gfx.setColor(this.pieceColor);
	
	// place a piece at x, startY
	setLocation(x, startY);
	drawPiece(gfx);
	// drop the piece from x, startY to x, endY
	for(int i = startY; i < endY; i++){
	    setLocation(x, i);
	    drawPiece(gfx);
	    for(int j = 0; j<10000000; j++){} // to make the animation
	    gfx.setColor(Color.black);
	    gfx.fillRect(x, i, 20, 20);
	    gfx.setColor(this.pieceColor);
	}
	// leave the piece sitting at location x, endY
	setLocation(x, endY);
	drawPiece(gfx);
	this.newOrSet = SET_PIECE;
    }

    /**
     * setLocation sets the x and y for the piece
     **/
    public void setLocation(int x, int y)
    {
	this.x = x;
	this.y = y;
    }

    /**
     * setSize sets the size of the piece this should be
	 * the smallest of the column width or the row height
     **/
    public void setSize(int size)
    {
	this.size = size;
    }

    /**
     * setColor sets the color of the piece
     **/
    public void setColor(Color pieceColor)
    {
	this.pieceColor = pieceColor;
    }

	/**
	 * setTexture sets the image to map to the piece
	 **/
	public void setTexture(TexturePaint imagePaint)
	{
		this.imagePaint = imagePaint;
	}



    /*
     * drawPiece draws the piece in its location
     **/
    public void drawPiece(Graphics g)
    {
		Graphics2D gfx = (Graphics2D)g;
		gfx.setPaint(pieceColor);
		if(this.imagePaint != null)
			gfx.setPaint(this.imagePaint);
		imageOval = new Ellipse2D.Double(this.x, this.y, size, size);
		gfx.fill(imageOval);

    }

}
