/*
 * EventHandler.java - handles all gui events.
 */

import java.awt.event.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.*;
import java.net.*;
import java.io.*;

class EventHandler
	implements MouseListener, ActionListener, ItemListener, WindowListener {
    private Socket sockToServer = null;   // socket to Server
    private GuiWindow gui;
    private ConnectN game;
    private String selectedSize = ""; // selected radio button board size.
    private DisplayPanel disPanel;
    private ButtonPanel btnPanel;
    private ChatPanel chatPanel;
    private PlayerCommunicator comLink = null;
    private JFrame userNameFrame;
    private JTextField userName1; // always used.
    private JTextField userName2; // only used for local game
    private JTextField hostNameText;
	private PlaySounds play = new PlaySounds();

    // constructor
    EventHandler(ConnectN gm, GuiWindow gui) {
	this.game = gm;
	this.gui = gui;
    }
    
    /**
     * actionPerformed(e) -- Gui event handler.
     */
    public void actionPerformed(ActionEvent e) {

	if(e.getActionCommand().equals("Network Game")) {
	    //System.out.println("User chose New Network Game");

	    // if we're currently connected to the client negotiatior server,
	    // and we're waiting for another player, then don't allow
	    // the user to connect again.
	    if(game.getOnlineState() == game.WAITING) {
		gui.updateChatWindow("New Network Game Error: You area already"+
				     " connected to the game server and " +
				     "waiting for another player.");
		return;
	    }

	    // start a Dialogue box as a separate thread that gets the desired
	    // user name. this class is the listener, and when the user clicks
	    // "ok", it will trigger a seperate else if clause of this method.
	    newNetGame();	    
	    
	}

	// handle new network game input validation (username and hostname)
	else if(e.getActionCommand().equals("okUserName") ||
		e.getActionCommand().equals("userNameText") ||
		e.getActionCommand().equals("hostNameText") ) {
	    String userName = userName1.getText().trim();
	    String hostName = hostNameText.getText().trim();
	    
	    // if either username was invalid.
	    if(userName.trim().length() < 1) {
		gui.updateChatWindow("Error Joining New Network Game: " +
				     "Your user name is invalid");
	    }

	    // if the hostname was invalid.
	    else if(hostName.trim().length() < 1) {
		gui.updateChatWindow("Error Joining New Network Game: " +
				     "Specified host name is invalid");
	    }

	    // if the username and hostname were valid:
	    else {	
		
		// if we were already connected to another player, disconnect.
		if(game.getOnlineState() == game.ONLINE) {
		    if(comLink != null) {		
			gui.updateChatWindow("You have disconnected from your opponnent.");
			comLink.sendLine("disconnect");
			comLink.disconnect();
			comLink = null;
		    }
		}
		// if the user name is more than 12 characters, truncate it 
		// to 12.
		if(userName.length() > 12) {
		    userName = userName.subSequence(0,12).toString();
		}
		// reset the board, gui, and chat panel and game states
		game.resetGame();
		gui.resetChatWindow();
		
		// set the hostname of the server
		game.setHostName(hostName.trim());

		// set the local user's username to the new value.
		game.setUserName1(userName.trim());
		//System.out.println("Setting this client's username to " +
		//		   userName);
		// go online.
		boolean isOnline = goOnline(hostName);
		if(!isOnline) {
		    game.setGameState(game.OUTGAME);
		    game.setOnlineState(game.OFFLINE);		    
		}
	    }
	    
	    // hide the dialogue box since they clicked OK
	    userNameFrame.setVisible(false);
	    userNameFrame.dispose();
	}

	// handle the dialogue box for new local game
	else if(e.getActionCommand().equals("Local Game")) {
	    //System.out.println("User chose New Local Game");

	    if(game.getOnlineState() == game.WAITING) {
		if(sockToServer != null) {
		    try { sockToServer.close(); }
		    catch(IOException f) {}
		    sockToServer = null;
		}
		game.setOnlineState(game.OFFLINE);
	    }	    

	    // print a dialogue that gets the players' names and the
	    // chosen board size.	   
	    newLocalGame();
	    
	}

	// do validation check on inputs for new local game
	else if(e.getActionCommand().equals("okLocUserName") ||
		e.getActionCommand().equals("locNameText1")  ||
		e.getActionCommand().equals("locNameText2") ) {
	    boolean inputOK = true; // assume input is valid

	    // if the first username is invalid
	    if( userName1.getText().trim().length() < 1 ){
		gui.updateChatWindow("Error Creating New Local Game: " +
				     "Player One's user name is invalid");
		inputOK = false;
	    }

	    // if the second username is invalid
	    if(userName2.getText().trim().length() < 1) {
		gui.updateChatWindow("Error Creating New Local Game: " +
				     "Player Two's user name is invalid");
		inputOK = false;
	    }

	    // if a board size wasn't selected.
	    if(selectedSize.length() < 1) {
		gui.updateChatWindow("Error Creating New Local Game: " +
				     "You didn't select a board size.");
		inputOK = false;
	    }

	    // if the user forgot to input valid text for either of the user
	    // names, or forgot to set a board size for the local game,
	    // reset all values for the next time they try to do so.
	    if(!inputOK) {
		userName1.setText("");
		userName2.setText("");
		selectedSize = "";
	    }

	    // if all values are valid, go here
	    if(inputOK) {

		// if we were already online, go offline
		if(game.getOnlineState() == game.ONLINE) {			
		    if(comLink != null) {
			comLink.sendLine("disconnect");
			comLink.disconnect();
			comLink = null;		   
		    }
		}
		// reset the board layout, chat window, other gui elements,
		// and game states.
		game.resetGame();
		gui.resetChatWindow();

		// set usernames chosen and board size.
		game.setUserName1(userName1.getText().trim());
		game.setUserName2(userName2.getText().trim());
		GameBoard board;
		if(selectedSize.equals("sevBySixButton")) {
		    board = new GameBoard(7, 6, 4);
		    game.setGameBoard(board);
		    gui.updateDisplayBoard();
		}
		else if (selectedSize.equals("sixByEitButton")) {
		    board = new GameBoard(6, 8, 4);
		    game.setGameBoard(board);
		    gui.updateDisplayBoard();
		}
		else if (selectedSize.equals("elvByTwvButton")) {
		    board = new GameBoard(11, 12, 5);
		    game.setGameBoard(board);
		    gui.updateDisplayBoard();
		}

		game.newLocalStates();
	    }

	    userName1.setText("");
	    userName2.setText("");
	    selectedSize = "";
	    userNameFrame.setVisible(false);
	    userNameFrame.dispose();  
	}

	// handle radio button selection of board size. 
	else if(e.getActionCommand().equals("sevBySixButton") ||
		e.getActionCommand().equals("sixByEitButton") ||
		e.getActionCommand().equals("elvByTwvButton") ) {

	    // save the chosen radio button board size.
	    selectedSize = e.getActionCommand();
	}

	// handle cancel gets pressed on any dialogue box.
	else if(e.getActionCommand().equals("cancelButton") ) {
	    userNameFrame.setVisible(false);
	    userNameFrame.dispose();
	}
	
	else if (e.getActionCommand().equals("Save Game")) {
	    //System.out.println("User chose Save Game");
	    gui.showSaveDialogue();	    
	}
	else if (e.getActionCommand().equals("Load Game")) {
	    //System.out.println("User chose Load Game");
	    gui.showLoadDialogue();
	}
	else if (e.getActionCommand().equals("High Scores")) {
	    //System.out.println("User chose High Scores");
	    gui.showStatsWindow();
	}
	else if (e.getActionCommand().equals("statsOk")) {
	    //System.out.println("User chose Stats Ok");
		gui.disposeStatsWindow();
		if(game.getGameState() == game.WINGAME
		   && game.getWinner() == game.getUserName1()) {
			gui.replayGame();
		}
	}

	else if (e.getActionCommand().equals("playAgain")) {
	    //System.out.println("User chose Play Again");
		gui.replayGameKill();
		game.handleReplayGame();
		if(comLink != null) {
			comLink.sendLine("replay");
   		}

	}
	else if (e.getActionCommand().equals("Disconnect")) {
	    //System.out.println("User chose Play Again");
		gui.replayGameKill();
		if(game.getOnlineState() == game.ONLINE) {			
		    if(comLink != null) {
			comLink.sendLine("disconnectreset");
			comLink.disconnect();
			comLink = null;		   
		    }
		}
		// reset the board layout, chat window, other gui elements,
		// and game states.
		game.resetGame();
	}

	else if (e.getActionCommand().equals("Exit")) {
	    //System.out.println("User chose Exit");
	    if(game.getOnlineState() == game.ONLINE) 
		    comLink.sendLine("disconnect");		
		System.exit(0);

	}
	else if (e.getActionCommand().equals("Lever Arm")) {
	    //System.out.println("User chose Lever Arm");
		if(game.getGameState() == game.WINGAME){		    
			if(game.getOnlineState() == game.OFFLINE) {
				//System.out.println("Empty the board");
				game.getGameBoard().empty();
				if(!game.isLeverPulled()) {
				    gui.animateLever();
				    game.setLeverPulled(true);
				    gui.resetChatWindow();
				    gui.updateDisplayBoard();
				    play.playAudio("spill.aif");
				}				
				else { gui.updateChatWindow("You already pulled the lever."); }
			}

			else if(game.getUserName1() == game.getWinner() ) {
				//System.out.println("Empty the board");
				game.getGameBoard().empty();
				if(!game.isLeverPulled()) {
				    gui.animateLever();
				    game.setLeverPulled(true);
				    gui.resetChatWindow();
				    gui.updateDisplayBoard();
				    if(comLink != null) {		    
				    comLink.sendLine("empty");
				    //System.out.println("Emptying other players board");
				    play.playAudio("spill.aif");
				    gui.showStatsWindow();
				    }
				}
				else { gui.updateChatWindow("You already pulled the lever."); }
			}
			else if(game.getUserName1() != game.getWinner() ) {
			    gui.updateChatWindow("You cannot pull the lever, you did not win.");
			}
		}else if(game.getGameState() == game.DRAWGAME){
			if(game.getOnlineState() == game.OFFLINE) {
				//System.out.println("Empty the board");
				game.getGameBoard().empty();
				if(!game.isLeverPulled()) {
				    gui.animateLever();
				    game.setLeverPulled(true);
				    gui.resetChatWindow();
				    gui.updateDisplayBoard();
				    play.playAudio("spill.aif");
				}
				else { gui.updateChatWindow("You already pulled the lever."); }
				
			}
			else {
				//System.out.println("Empty the board");
				game.getGameBoard().empty();
				if(!game.isLeverPulled()) {
				    gui.animateLever();
				    game.setLeverPulled(true);
				    gui.resetChatWindow();
				    gui.updateDisplayBoard();
				    if(comLink != null) {		    
					comLink.sendLine("empty");
					//System.out.println("Emptying other players board");
				    }
				    play.playAudio("spill.aif");
				}
				else { gui.updateChatWindow("You already pulled the lever."); }
				
				
			}
		}else
		gui.updateChatWindow("You must win a match before you are allowed to" +
				     " clear the board.");
		
	}

	// else if the player pressed button 1 - 12.
	else if ( e.getActionCommand().equals("1") || 
		  e.getActionCommand().equals("2") ||
		  e.getActionCommand().equals("3") ||
		  e.getActionCommand().equals("4") ||
		  e.getActionCommand().equals("5") ||
		  e.getActionCommand().equals("6") ||
		  e.getActionCommand().equals("7") || 
		  e.getActionCommand().equals("8") || 
		  e.getActionCommand().equals("9") || 
		  e.getActionCommand().equals("10")|| 
		  e.getActionCommand().equals("11") ) {		  

	    // compute the column that was selected for the move
	    // subtract 1 because the GameBoard columns are from 0...n-1.
	    int column = ( Integer.parseInt(e.getActionCommand().trim()) - 1 );

	    //System.out.println("User pressed button " + (column + 1));

	    // test to see if we're in a game. if not, don't make the move.
	    if( game.getGameState() != game.INGAME ) {
		gui.updateChatWindow("Piece placement not allowed: You're not " 
				 + " currently in a game!");
		return;
	    }

	    // if we're online, it has to be my turn in order to make
	    // a move. if we're offline, there is no "my" from the
	    // client's point of view and a move can always be made locally.
	    if( (game.getOnlineState() == game.ONLINE) &&
	        (game.getPlayerNum() != game.getPlayerState()) ) {
		gui.updateChatWindow("Piece placement not allowed: It's not " +
				     "your turn!");
		return;
	    }

	    // call updateGame, which places the piece on the board if it is
	    // valid. if it succeeded and we're online, send the move to
	    // the other player. 
	    if( (game.updateGame(column)) && 
		(game.getOnlineState() == game.ONLINE) ) {
			comLink.sendLine("move " + column);
	    }
	}

	// otherwise we treat the command as a chat line. this is actually
	// okay, (ignore the comment block that used to be here). we don't
	// need a separate EventHandler for chat functionality because
	// we used setActionCommand to make the actionCommands for the
	// buttons pretty obscure :).

	else {
	    //System.out.println("chat line: " + e.getActionCommand());
	    if(game.getOnlineState() == game.ONLINE) {
		String line = "<" + game.getUserName1() + "> " 
		              + e.getActionCommand();
		// update the local gui with the chat statement.
		gui.updateChatWindow(line);

		// concat "chat " onto the line so the other player knows
		// its a chat line.
		line = "chat " + line;

		// send it to the other player.
		comLink.sendLine(line);		
	    }
	    else { gui.updateChatWindow("Sorry, Chatting not allowed when " +
					"not connected to another player.");
	    }
	}

    }
    
    // when the user clicks file-> new game -> new local game, this panel
    // pops up to get game parameter inputs. called from ActionEvent
    private void newLocalGame() {
	JFrame.setDefaultLookAndFeelDecorated(true);
		
	userNameFrame = new JFrame("User / Board Configuration");
	userNameFrame.setBackground(ConnectN.GUICOLOR);
	userNameFrame.setLocationRelativeTo(gui.getGuiFrame());
	JPanel thePanel = new JPanel(new BorderLayout());
	thePanel.setBackground(ConnectN.GUICOLOR);
	JPanel userPanel = new JPanel(new GridLayout(3,2));
	userPanel.setBackground(ConnectN.GUICOLOR);
	JPanel buttonPanel = new JPanel(new GridLayout(0,1));
	buttonPanel.setBackground(ConnectN.GUICOLOR);
	JButton cancel = new JButton("Cancel");
	cancel.setBackground(ConnectN.GUICOLOR);
	cancel.setActionCommand("cancelButton");
	cancel.addActionListener(this);

	JButton ok = new JButton("OK");
	ok.setBackground(ConnectN.GUICOLOR);
	ok.setActionCommand("okLocUserName");
	ok.addActionListener(this);
	
	userName1 = new JTextField("", 10);
	userName1.setBackground(ConnectN.TEXTAREACOLOR);
	userName1.setActionCommand("locNameText1");
	userName1.addActionListener(this);
	
	userName2 = new JTextField("", 10);
	userName2.setBackground(ConnectN.TEXTAREACOLOR);
	userName2.setActionCommand("locNameText2");
	userName2.addActionListener(this);

	JLabel userLabel1 = new JLabel("Enter Player One's user name: ");
	userLabel1.setLabelFor(userName1);
	userLabel1.setBackground(ConnectN.GUICOLOR);	

	JLabel userLabel2 = new JLabel("Enter Player Two's user name: ");
	userLabel2.setLabelFor(userName2);
	userLabel2.setBackground(ConnectN.GUICOLOR);	

	JLabel radioLabel = new JLabel("Select a Board Size: ");
	// radio buttons
	JRadioButton sevBySixButton = new JRadioButton("7x6");
	sevBySixButton.setBackground(ConnectN.GUICOLOR);
	sevBySixButton.setActionCommand("sevBySixButton");
	sevBySixButton.addActionListener(this);

	JRadioButton sixByEitButton = new JRadioButton("6x8");       
	sixByEitButton.setBackground(ConnectN.GUICOLOR);
	sixByEitButton.setActionCommand("sixByEitButton");
	sixByEitButton.addActionListener(this);

	JRadioButton elvByTwvButton = new JRadioButton("11x12");
	elvByTwvButton.setBackground(ConnectN.GUICOLOR);
	elvByTwvButton.setActionCommand("elvByTwvButton");
	elvByTwvButton.addActionListener(this);
	
	// do drop down menu for board sizes.
	ButtonGroup group = new ButtonGroup();
	group.add(sevBySixButton);
	group.add(sixByEitButton);
	group.add(elvByTwvButton);
	
	buttonPanel.add(radioLabel);
	buttonPanel.add(sevBySixButton);
	buttonPanel.add(sixByEitButton);
	buttonPanel.add(elvByTwvButton);
	buttonPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	userPanel.add(userLabel1);
	userPanel.add(userName1);
	userPanel.add(userLabel2);
	userPanel.add(userName2);
	userPanel.add(cancel);
	userPanel.add(ok);

	thePanel.add(userPanel, BorderLayout.WEST);
	thePanel.add(buttonPanel, BorderLayout.EAST);
	thePanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	userNameFrame.getContentPane().add(thePanel, BorderLayout.CENTER);
	
	userNameFrame.pack();
	userNameFrame.show();	
	
    }
    
    // when the user clicks file-> new game -> new network game, this panel
    // pops up to get game parameter inputs. called from ActionEvent
    private void newNetGame() {
	JFrame.setDefaultLookAndFeelDecorated(true);
	
	userNameFrame = new JFrame("User / Board Configuration");
	userNameFrame.setBackground(ConnectN.GUICOLOR);
	userNameFrame.setLocationRelativeTo(gui.getGuiFrame());
	JPanel userPanel = new JPanel(new GridLayout(3,2));
	userPanel.setBackground(ConnectN.GUICOLOR);
	JButton cancel = new JButton("Cancel");
	cancel.setBackground(ConnectN.GUICOLOR);
	cancel.setActionCommand("cancelButton");
	cancel.addActionListener(this);

	JButton ok = new JButton("OK");
	ok.setBackground(ConnectN.GUICOLOR);
	ok.setActionCommand("okUserName");
	ok.addActionListener(this);

	hostNameText = new JTextField("", 10);
	hostNameText.setBackground(ConnectN.TEXTAREACOLOR);
	hostNameText.setActionCommand("hostNameText");
	hostNameText.addActionListener(this);

	userName1 = new JTextField("", 10);
	userName1.setBackground(ConnectN.TEXTAREACOLOR);
	userName1.setActionCommand("userNameText");
	userName1.addActionListener(this);

	JLabel userNameLabel = new JLabel("Enter a user name: ");
	userNameLabel.setBackground(ConnectN.GUICOLOR);
	userNameLabel.setLabelFor(userName1);

	JLabel hostNameLabel = new JLabel("Enter a host name: ");
	hostNameLabel.setBackground(ConnectN.GUICOLOR);
	hostNameLabel.setLabelFor(hostNameText);

	userPanel.add(userNameLabel);
	userPanel.add(userName1);
	userPanel.add(hostNameLabel);
	userPanel.add(hostNameText);
	userPanel.add(cancel);
	userPanel.add(ok);
	
	userNameFrame.getContentPane().add(userPanel, BorderLayout.CENTER);
	userNameFrame.setSize(50,50);	
	
	userNameFrame.pack();
	userNameFrame.show();	
    }

    /**
     * This method is used for any menu actions what change
     * states.
     **/	
    public void itemStateChanged(ItemEvent e) {
        JMenuItem source = (JMenuItem)(e.getSource());
		source.setBackground(ConnectN.GUICOLOR);
        String s = "Item event detected."
	    + "\n"
	    + "    Event source: " + source.getText()
	    + " (an instance of " + getClassName(source) + ")"
	    + "\n"
	    + "    New state: "
	    + ((e.getStateChange() == ItemEvent.SELECTED) ?
	       "selected":"unselected");
		//System.out.println(s);
    }
    
    // Returns just the class name -- no package info.
    protected String getClassName(Object o) {
        String classString = o.getClass().getName();
        int dotIndex = classString.lastIndexOf(".");
        return classString.substring(dotIndex+1);
    }

    /**
     * MouseListener methods.
     **/    
    public void mousePressed(MouseEvent event) {       
	GameBoard board = game.getGameBoard();

	//make the mouse pressed select a column
        int x = event.getX();   // Save the x coord of the click
        int y = event.getY();   // Save the y coord of the click

	// Get the window size
	int windowX = gui.getPaneSizeX();
	int windowY = gui.getPaneSizeY();
	// Get the board size
	int boardColumn = board.getWidth();
	int boardRow = board.getHeight();
	
	// Find which column the mouse clicked on
	int column = ( x / (windowX/ boardColumn));
	
        String s = "Mouse event detected\n"
	    + "    Mouse Pressed at: "
	    + x + "," + y
	    + "\n"
	    +  column;
        //System.out.println(s);	

	// test to see if we're in a game. if not, don't make the move.
	if( game.getGameState() != game.INGAME ) {
	    gui.updateChatWindow("Piece placement not allowed: You're not " +
				 " currently in a game!");
	    return;
	}
	
	// if we're online, it has to be my turn in order to make
	// a move. if we're offline then we're in a local game, and there is 
	// no "my" from the client's point of view and a move can always be 
	// made locally.
	if( (game.getOnlineState() == game.ONLINE) &&
	    (game.getPlayerNum() != game.getPlayerState()) ) {
	    gui.updateChatWindow("Piece placement not allowed: It's not your " +
				 "turn!");
	    return;
	}	

	// otherwise call updateGame, which places the piece on the board if 
	// it is valid. if the call  succeeded and we're online, 
	// send the move to the other player. 
	if( (game.updateGame(column)) && 
	    (game.getOnlineState() == game.ONLINE) ) {
	    comLink.sendLine("move " + column);
	}
    }

    // method to start a PlayerCommunicator and connect to the client negotiator
    // server. called from ActionEvent.
    public boolean goOnline(String host) {
	int MAXPORTS = 7789;
	boolean isPortOpened = false;  // do we have access to the port?
	int portNum = 7777; // default port to connect to.

	while(isPortOpened == false) {
	    try { 
		game.setOnlineState(game.WAITING);
		gui.updateDisplayBoard();
		sockToServer = new Socket(host, portNum);		
		isPortOpened = true;
		
		comLink = new PlayerCommunicator(game, gui, sockToServer);
		comLink.start();
		return(true);		
	    }

	    catch(IOException e) {
		//System.out.println("Could not connect on port: " + portNum);
		portNum = portNum + 2; 
		//System.out.println("Trying " + portNum);
	    }
	    if(portNum > MAXPORTS) {
		gui.updateChatWindow("Network Error: Could not connect to " +
				     "server.");
		game.setOnlineState(game.OUTGAME);
		gui.updateDisplayBoard();
		sockToServer = null;
	      return false;
	  }
	}
	return (false);
    }


    public void mouseClicked(MouseEvent event) {
		// Empty
    }
    public void mouseEntered(MouseEvent event) {
		// Empty
    }
    public void mouseExited(MouseEvent event) {
		// Empty
    }

    public void mouseReleased(MouseEvent event) {
		// Empty
    }


	/**
	 * Window Listener
	 **/
	public void windowClosing(WindowEvent e)
	{ 
		//System.out.println("WindowFocusListener method called: windowClosing.");
		if(game.getOnlineState() == game.ONLINE)
			comLink.sendLine("disconnect");
		System.exit(0);
	}
    public void windowClosed(WindowEvent e) {
        //This will only be seen on standard output.
        //System.out.println("WindowListener method called: windowClosed.");
    }

    public void windowOpened(WindowEvent e) {
        //System.out.println("WindowListener method called: windowOpened.");
    }

    public void windowIconified(WindowEvent e) {
        //System.out.println("WindowListener method called: windowIconified.");
    }

    public void windowDeiconified(WindowEvent e) {
        //System.out.println("WindowListener method called: windowDeiconified.");
    }

    public void windowActivated(WindowEvent e) {
        //System.out.println("WindowListener method called: windowActivated.");
    }

    public void windowDeactivated(WindowEvent e) {
		// System.out.println("WindowListener method called: windowDeactivated.");
    }

    public void windowGainedFocus(WindowEvent e) {
        //System.out.println("WindowFocusListener method called: windowGainedFocus.");
    }

    public void windowLostFocus(WindowEvent e) {
		// System.out.println("WindowFocusListener method called: windowLostFocus.");
    }

    public void windowStateChanged(WindowEvent e) {
		// System.out.println(
        //  "WindowStateListener method called: windowStateChanged." + e);
	}
}



