/**
* Gui is the module which does all of the displays
* for ConnectN. It creates a visual representation
* of the game board.
* 
* Pardon Our French
* date created: 5/7/2004
* last revision: 5/16/2004
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class Gui extends JFrame
    implements MouseListener,
    WindowListener
{
    private static final int GAME_STATE_INTRO = 0;
    private static final int GAME_STATE_SETUP = 1;
    private static final int GAME_STATE_PLAYING = 2;
    private static final int GAME_STATE_GAMEOVER = 3;
    private static final int RED_GRID_X = 5;
    private static final int RED_GRID_Y = 5;
    private static final int WINDOW_WIDTH = 600;
    private static final int WINDOW_HEIGHT = 700;
    private static final int DEFAULT_COLUMN = 7;
    private static final int DEFAULT_ROW = 6;
    private static final int DEFAULT_CONNECT = 4;

    ///////////////////////////////////////////////////
    // WindowListener Interface
    ///////////////////////////////////////////////////
    public void windowActivated(WindowEvent event) {}
    public void windowClosed(WindowEvent event) {}
    public void windowDeactivated(WindowEvent event) {}
    public void windowDeiconified(WindowEvent event) {}
    public void windowIconified(WindowEvent event) {}
    public void windowOpened(WindowEvent event) {}

    // instance variables
    private int gameState;
    private DisplayBoard disBoard;
    private Piece redPiece;
    private Vector redPiecesList;
    private Vector blackPiecesList;

    private int columns;
    private int rows;
    private int drawingBoardWidth;
    private int drawingBoardHeight;

    // these will be the variables which are called
    // until then I will comment them out so I can
    // still compile

    private GameBoard gameBoard;
    // private Chat chatDisplay;
    // private Network connection;
    // private Popup splashScreen;
    // private Popup winner;

    /**
     * Constructor for objects of class Gui
     */
    public Gui()
    {
	this.setSize(WINDOW_HEIGHT, WINDOW_WIDTH);
	this.show();
	this.addMouseListener(this);
	this.addWindowListener(this);
	this.gameState = GAME_STATE_INTRO;
	this.gameBoard = new GameBoard(DEFAULT_COLUMN, DEFAULT_ROW,
				   DEFAULT_CONNECT);
        this.disBoard = new DisplayBoard(DEFAULT_COLUMN, DEFAULT_ROW);
	this.redPiece = new Piece();
	this.redPiecesList = new Vector();
	this.redPiecesList.add(redPiece);
    }

    /**
     * Make program exit when window closes.
     */
    public void windowClosing(WindowEvent event) {
	this.dispose();
    }


    /**
     * Entry point for code execution.
     */
    public static void main(String[] args)
    {
	new Gui();
    }

    /**
     * Draws the 'Welcome to ConnectN' screen.
     */
    private void drawSplashScreen() {
	Container clientArea = this.getContentPane();
	int width = clientArea.getWidth();
	int height = clientArea.getHeight();
	Graphics gfx = clientArea.getGraphics();
	gfx.fillRect(0, 0, width, height);
	gfx.setColor(Color.green);
	gfx.drawString("Welcome to ConnectN", 236, 225);
	gfx.setColor(Color.gray);
	gfx.drawString("(click mouse to continue)", 228, 275);
    }
    
    /**
     * Draws the game window.
     */
    public void paint(Graphics gfx) {
	if (this.gameState == GAME_STATE_INTRO) {
	    this.drawSplashScreen();
	}
	else if (this.gameState == GAME_STATE_SETUP) {
	    this.drawGrids();
	    this.drawPieces();
	}
    }
    
    /**
     * Draw the game pieces
     **/
    private void drawPieces() {

	// Get an object representing the area within the window borders.
	Container clientArea = this.getContentPane();
	
	// Get the Graphics object associated with the client area.
	Graphics gfx = clientArea.getGraphics();
	
	// Get the size of the client area.
	int width = clientArea.getWidth();
	int height = clientArea.getHeight();
	
	// Set the piece color
	this.redPiece.setColor(Color.red);
	// Set the piece xy
	//this.redPiece.setLocation(RED_GRID_X + 10, RED_GRID_Y + 10);
	// Draw the pieces
	this.redPiece.drawPiece(gfx);
	// Move the piece
	//this.redPiece.movePiece(gfx, RED_GRID_X + 10, RED_GRID_Y + 10,
	//			RED_GRID_Y + 100);


    }
	

    /**
     * Draw the game grids.
     */
    private void drawGrids() {
	// Get an object representing the area within the window borders.
	Container clientArea = this.getContentPane();
	
	// Get the Graphics object associated with the client area.
	Graphics gfx = clientArea.getGraphics();
	
	// Get the size of the client area.
	int width = clientArea.getWidth();
	int height = clientArea.getHeight();
	
	// Fill the background with black.
	gfx.setColor(Color.black);
	gfx.fillRect(0, 0, width, height );
	
	// Draw the grid.
	this.disBoard.drawGrid(gfx, Color.yellow, RED_GRID_X, RED_GRID_Y,
			       width, height);

   }
    


        
    /**
     * MouseListener methods.
     */
    
    public void mouseClicked(MouseEvent event) {
	// Empty
    }
    public void mouseEntered(MouseEvent event) {
	// Empty
    }
    public void mouseExited(MouseEvent event) {
	// Empty
    }
    public void mousePressed(MouseEvent event) {
	//make the mouse pressed move the circle
	    this.redPiece.setLocation(event.getX(), event.getY());
	    this.repaint();
    }
    public void mouseReleased(MouseEvent event) {
	this.gameState = GAME_STATE_SETUP;
	this.repaint(); // Redraws the game screen.
    }


}


