/**
 * LeverPanel.java - this is the panel along the right side of the board.
 * This includes the lever arm and a display of whos turn it currently is.
 **/
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


public class LeverPanel extends JPanel
	implements Runnable{
	private EventHandler listener;
	private JButton buttonLeverArm;
	private PlayerPanel whichPlayerPanel;
	private String leverArmPicFile;
	private JPanel leverPanel;
	ImageIcon leverIcon;
	Thread animationThread;

	private GameBoard board;

	public LeverPanel(ConnectN game, EventHandler eventlistener)
	{
		this.setLayout( new GridLayout(3,1));
		this.setBackground(ConnectN.GUICOLOR);
		this.listener = eventlistener;
	
		// Initialize whichPlayerPanel
		whichPlayerPanel = new PlayerPanel(game, listener);
		whichPlayerPanel.setPreferredSize(new Dimension(100, 50));

		//Add the PlayerPanel
		add(whichPlayerPanel);

		// Initialize placeholderPanel
		JPanel placeholderPanel = new JPanel();
		placeholderPanel.setBackground(ConnectN.GUICOLOR);
		
		//Add the placeholder
		add(placeholderPanel);

		// Initialize buttons
		leverArmPicFile = "leverarm1.gif";
		leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm = new JButton(leverIcon);
		buttonLeverArm.setBackground(ConnectN.GUICOLOR);
		buttonLeverArm.setBorderPainted(false);
		buttonLeverArm.setActionCommand("Lever Arm");
		

		// Make buttons use the ActionListener
		buttonLeverArm.addActionListener(listener);

		//Add the buttons
		add(buttonLeverArm);

	}

    /**
	 * createImageIcon Returns an ImageIcon, or null if the
	 * path was invalid.
	 **/
    protected static ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = LeverPanel.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            System.err.println("Couldn't find file: " + path);
            return null;
        }
    }

	/**
	 * Initiates animation
	 **/
	public void animateLever(){
		animationThread = new Thread(this);
		animationThread.start();
	}

	public void paintComponent(Graphics g) {
		buttonLeverArm.setIcon(leverIcon);
	}

	public void run() {
		long time = System.currentTimeMillis();
		leverArmPicFile = "leverarm1.gif";
		leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm.setIcon(leverIcon);
		paintComponent(getGraphics());

		try {
			Thread.sleep(Math.max(80,time - System.currentTimeMillis()));
		}catch (InterruptedException e) {
			System.err.println(e);
			}

		leverArmPicFile = "leverarm2.gif";
		leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm.setIcon(leverIcon);
		paintComponent(getGraphics());

		try {
			Thread.sleep(Math.max(80,time - System.currentTimeMillis()));
		}catch (InterruptedException e) {
			System.err.println(e);
			}

		leverArmPicFile = "leverarm3.gif";
		leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm.setIcon(leverIcon);
		paintComponent(getGraphics());

		try {
			Thread.sleep(Math.max(80,time - System.currentTimeMillis()));
		}catch (InterruptedException e) {
			System.err.println(e);
			}

		leverArmPicFile = "leverarm4.gif";
		leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm.setIcon(leverIcon);
		paintComponent(getGraphics());

		try {
			Thread.sleep(Math.max(90,time - System.currentTimeMillis()));
		}catch (InterruptedException e) {
			System.err.println(e);
			}
		leverArmPicFile = "leverarm3.gif";
		leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm.setIcon(leverIcon);
		paintComponent(getGraphics());

		try {
			Thread.sleep(Math.max(70,time - System.currentTimeMillis()));
		}catch (InterruptedException e) {
			System.err.println(e);
			}


		leverArmPicFile = "leverarm2.gif";
		leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm.setIcon(leverIcon);
		paintComponent(getGraphics());

		try {
			Thread.sleep(Math.max(50,time - System.currentTimeMillis()));
		}catch (InterruptedException e) {
			System.err.println(e);
			}

		leverArmPicFile = "leverarm1.gif";
	    leverIcon = createImageIcon(leverArmPicFile);
		buttonLeverArm.setIcon(leverIcon);

	}
}
	
class PlayerPanel extends JPanel{
	private EventHandler listener;
	private GameBoard board;
	private ConnectN game;
        private String curPlayer = "";
        private String winner = "";
        private String netState = "";

	public PlayerPanel(ConnectN game, EventHandler eventlistener){
		this.game = game;
		this.setBackground(ConnectN.GUICOLOR);
	}

	/** 
	 * Paints the inside of the panel. It draws the board.
	 **/
	public void paintComponent(Graphics g) {
		//board = game.getGameBoard();
		//super.paintComponent(g);
		Graphics2D gfx = (Graphics2D)g;
		
		// Get the size of the client area.
		int width = super.getWidth();
		int height = super.getWidth();

		gfx.setColor(ConnectN.GUICOLOR);
				    
		// if we're in a game, print whose turn it is.
		if(game.getGameState() == game.INGAME ){
		    if(game.getOnlineState() == game.ONLINE) { netState = "Online Game"; }
		    else { netState = "Local Game"; }

		    // use getCurPlayer to get the current player
		    curPlayer = game.getCurPlayer();

		    // if it is player 1's turn, draw red
		    if( game.getPlayerState() == game.PLAYER_ONE_TURN ) {
			gfx.setColor(Color.red);
			gfx.fillRect(0,0, width, height);
			gfx.setColor(Color.white);
			gfx.drawString(netState, 15,15 );
			gfx.drawString((curPlayer + "'s"), 10, 40);
			gfx.drawString("Turn", 10, 55);
		    }

		    // if it is player 2's turn, draw black
		    else if(game.getPlayerState() == game.PLAYER_TWO) {
			gfx.setColor(Color.black);
			gfx.fillRect(0,0, width, height);
			gfx.setColor(Color.white);
			gfx.drawString(netState, 15,15 );
			gfx.drawString((curPlayer + "'s"), 10, 40);
			gfx.drawString("Turn", 10, 55);
		    }		
		}
	

		// if we're in a WINGAME state, someone won. Whosever turn it was when the WINGAME
		// state was triggered is the winner.	       
		else if(game.getGameState() == game.WINGAME) {

		    // call getWinner()
		    winner = game.getWinner();
		    
		    if(game.getPlayerState() == game.PLAYER_ONE_TURN) {
			gfx.setColor(Color.red);
			gfx.fillRect(0,0, width, height);
			gfx.setColor(Color.white);
			gfx.drawString(winner, 10, 40);
			gfx.drawString("Won!", 10, 55);
		    }

		    // if player2 won, draw black
		    else if(game.getPlayerState() == game.PLAYER_TWO_TURN) {
			gfx.setColor(Color.black);
			gfx.fillRect(0,0, width, height);
			gfx.setColor(Color.white);
			gfx.drawString(winner, 10, 40);
			gfx.drawString("Won!", 10, 55);			
		    }
		}
		
		// if we're waiting for a nother player to join the game, print a descriptive message.
		else if(game.getOnlineState() == game.WAITING) {
		    gfx.setColor(Color.blue);
		    gfx.fillRect(0,0, width, height);
		    gfx.setColor(Color.white);
		    gfx.drawString("Waiting for", 10, 40);
		    gfx.drawString("other player", 10, 55);
		}

		else if(game.getGameState() == game.DRAWGAME) {
		    gfx.setColor(Color.blue);
		    gfx.fillRect(0,0, width, height);
			gfx.setColor(Color.white);
		    gfx.drawString("It's a Draw!", 10, 40);
		}

		else {
		    gfx.setColor(new Color(0, 150, 0));
		    gfx.fillRect(0,0, width, height);
		    gfx.setColor(Color.white);
		    gfx.drawString("No Game", 10,15 );
		    gfx.drawString("In Progress", 10,30 );
		}
	}
}	

