/**
 * SplashWindow is the class which makes a window frame for the splash
 * screen.
 **/
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.event.*;

public class SplashWindow{
	
	private EventHandler listener;
	private JFrame splashFrame;
	private SplashPanel splashMainPanel;
	private JPanel splashButtonPanel;
	private PlaySounds play = new PlaySounds();
	
	public SplashWindow( EventHandler listener ){

		this.listener = listener;
	    //Make sure we have nice window decorations.
	    JFrame.setDefaultLookAndFeelDecorated(true);

		splashFrame = new JFrame("ConnectN");
		splashFrame.setBackground(ConnectN.GUICOLOR);
	    splashFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);


		Container splashFrameContent = splashFrame.getContentPane();
		splashFrameContent.setBackground(ConnectN.GUICOLOR);
		splashFrameContent.setLayout(new BoxLayout(splashFrameContent,
												   BoxLayout.Y_AXIS));


		splashMainPanel = new SplashPanel();
		splashMainPanel.setBackground(ConnectN.GUICOLOR);

		Graphics2D gfx = (Graphics2D)splashFrameContent.getGraphics();


		//System.out.println("The SplashWindow has opened");
		splashButtonPanel = new JPanel();
		splashButtonPanel.setBackground(ConnectN.GUICOLOR);
		JButton btnNetwork = new JButton("Network Game");
		btnNetwork.setBackground(ConnectN.GUICOLOR);
		JButton btnLocal = new JButton("Local Game");
		btnLocal.setBackground(ConnectN.GUICOLOR);
		JButton btnLoad = new JButton("Load Game");
		btnLoad.setBackground(ConnectN.GUICOLOR);


		btnNetwork.addActionListener(listener);
		btnLocal.addActionListener(listener);
		btnLoad.addActionListener(listener);

		splashButtonPanel.add(btnNetwork);
		splashButtonPanel.add(btnLocal);
		splashButtonPanel.add(btnLoad);

		splashFrameContent.add(splashMainPanel);
		splashFrameContent.add(splashButtonPanel);
		splashFrame.setSize(300, 300);
		splashFrame.pack();
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		Rectangle window = splashFrame.getBounds();
		splashFrame.setLocation(
				 ((screen.width - window.width) /2),
				 ((screen.height - window.height) /2));

		play.playAudio("welcome.aif");


	}

	public void setVisible(boolean state){
		splashFrame.setVisible(state);
		if(state == false){
			splashFrame.dispose();
		}
	}


}

class SplashPanel extends JPanel
{

	private TexturePaint imagePaint;
	private BufferedImage image;
	private String imageFile = "connect4.jpg";

	SplashPanel(){
		setPreferredSize(new Dimension(360, 360));
		image = ImageUtilities.getBufferedImage(imageFile, this);

	}
	
	public void paintComponent(Graphics g){
		this.setBackground(ConnectN.GUICOLOR);
		Graphics2D gfx = (Graphics2D)g;
		imagePaint = new TexturePaint(image,
									  new Rectangle(0, 0,
													this.getWidth(),
													this.getHeight())); 
 		gfx.setPaint(imagePaint);
		Rectangle imageRect = new Rectangle(0, 0,
											this.getWidth(),
											this.getHeight());
		gfx.fill(imageRect);
		
	}
}
