/**
 * StatsWindow is the class which makes a window frame for the stats
 * screen.
 **/
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class StatsWindow {	
    private ConnectN game;
    private EventHandler listener;
    private JFrame statsFrame;
    private StatsPanel statsMainPanel;
    private JPanel statsButtonPanel;
    private Vector statInfo;
    private Socket sockToServer = null;
    private BufferedReader serverInput = null;
    private PrintWriter serverOutput = null;

    public StatsWindow( ConnectN game, EventHandler listener ){
	statInfo = new Vector(100, 100);
	this.game = game;
	this.listener = listener;
    }
    
    public boolean buildStatsWindow() {
	//Make sure we have nice window decorations.
	JFrame.setDefaultLookAndFeelDecorated(true);
	
	statsFrame = new JFrame("ConnectN Winners");
	statsFrame.setBackground(ConnectN.GUICOLOR);
	
	
	Container statsFrameContent = statsFrame.getContentPane();
	statsFrameContent.setBackground(ConnectN.GUICOLOR);
	statsFrameContent.setLayout(new BoxLayout(statsFrameContent,
						  BoxLayout.Y_AXIS));
	statsMainPanel = new StatsPanel();
	
	Graphics2D gfx = (Graphics2D)statsFrameContent.getGraphics();
	
	
	//System.out.println("The StatsWindow has opened");
	statsButtonPanel = new JPanel();
	statsButtonPanel.setBackground(ConnectN.GUICOLOR);
	statsButtonPanel.setLayout(new BorderLayout());
	
	// get the list of winners
	if( !getStatsFromServer() ) { return false; } 


	String[] columnNames = { "Date of Game ", "User Name", "Goal To Win",
				 "Number of Moves" };
	 
	int numStats = statInfo.size();
	Object [][] data = new Object[numStats][4];
	String curStat = "";
	StringTokenizer st = null;
	String userName = "";
	int j = 0;
	for(int i = 0; i < statInfo.size(); i++) {
	    j=0;
	    curStat = (String) statInfo.get(i);
	    st = new StringTokenizer(curStat);

	    // put the date into the data object
	    data[i][j] = new String(st.nextToken());
	    j++;

	    // temporarily state the goal
	    int goal = Integer.parseInt(st.nextToken().trim());
	    // temporarily store the num of moves
	    int numMoves = Integer.parseInt(st.nextToken().trim());
	
	    userName = "";
	    // put the username into the data object
	    while(st.hasMoreElements()) {
		userName = userName + " " + st.nextToken();
	    }	    	    
	    data[i][j] = new String(userName);
	    j++;

	    // now put the goal and num of moves into the string.
	    data[i][j] = new Integer(goal);
	    j++;

	    data[i][j] = new Integer(numMoves);	
	}

	final JTable table = new JTable(data, columnNames);
	table.setBackground(ConnectN.TEXTAREACOLOR);
	table.setPreferredScrollableViewportSize(new Dimension(400, 200));
	JScrollPane scrollPane = new JScrollPane(table,
		JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

	statsButtonPanel.add(scrollPane, BorderLayout.CENTER);       	

	JButton btnStatsOk = new JButton("Ok");
	btnStatsOk.setBackground(ConnectN.GUICOLOR);
	btnStatsOk.setActionCommand("statsOk");
	btnStatsOk.addActionListener(listener);
	statsButtonPanel.add(btnStatsOk, BorderLayout.SOUTH);
	
	statsFrameContent.add(statsMainPanel);
	statsFrameContent.add(statsButtonPanel);
	statsFrame.pack();
	statsFrame.show();
	
	return true;
    }

    public void setVisible(boolean state) {
	statsFrame.setVisible(state);
	if(state == false){
	    statsFrame.dispose();
	}
    }
    
    public boolean sendStatToServer() {
	String userName = "";
	String boardSize = "";
	int numMoves = -1;  // number of moves to win
	int goal = -1;  // goal of game
	GameBoard board = game.getGameBoard();
	String stat = "";
	
	try {
	    if(!connectToServer()) { return false; }

	    // get board size
	    if( (board.getWidth() == 7) && (board.getHeight() == 6) ) 
		boardSize = "7x6";
	    else if( (board.getWidth() == 8) && (board.getHeight() == 6)) 
		boardSize = "8x6";
	    else if( (board.getWidth() == 11) && (board.getHeight() == 12)) 
		boardSize = "11x12";
	    
	    // get the goal of the win
	    goal = board.getGoal();
	    
	    // get who won and the number of moves they performed and
	    // their username.
	    if( game.getWinner() == game.getUserName1() ) {
		numMoves = game.getNumP1Moves();
		userName = game.getUserName1();
	    }
	    else {		
		numMoves = game.getNumP2Moves();
		userName = game.getUserName2();
	    }
	    
	    // begin stat server communication
	    serverOutput.println("stats update");
	    serverOutput.flush();
	    
	    //if server replies ok
	    if(serverInput.readLine().equals("ok")) {
		//send the stat of the win
		stat = "" + goal + " " + numMoves + " " + userName;
		//System.out.println("sending: " + stat);
		serverOutput.println(stat);
		serverOutput.flush();
	    }

	    disconnectFromServer();

	}
	
	catch (IOException e) { 
	    System.err.println("sendStatToServer threw IOException."); 
	    return false;
	}

	return true;
    }

    public boolean getStatsFromServer() {	
	String stat = "";
	statInfo = new Vector(100,100);

	try {
	    if( !connectToServer() ) { return false; }

	    // begin stats receival
	    serverOutput.println("stats receive");
	    serverOutput.flush();
	    
	    // if server replies ok
	    if(serverInput.readLine().equals("ok")) {
		
		// while there are still stats to process
		for( stat = serverInput.readLine(); !stat.equals("done");
		     stat = serverInput.readLine() ) {
		    //System.out.println("received line: " + stat + 
			//	       "from server");
		    //stat = formatStatLine(stat);
		    //System.out.println("formatted stat: " + stat);

		    //stat = stat.replaceAll(" ", "\t");
		    // append the stats to the textArea.
		    //System.out.println("inserting: " + stat);
		    statInfo.add(stat);
		}
	    }

	    disconnectFromServer();
	}

	catch (IOException e) { 
	    System.err.println("getStatsFromServer threw IOException.");
	    return false;
	}

	return true;	
    }

    private String formatStatLine(String stat) {
	StringTokenizer st = null;
	String curToken = "";
	String userName = "";
	String frmtdStat = "";

	// tokenize the line and check number of tokens. 
	st = new StringTokenizer(stat);

	String dateString = st.nextToken();

	// store the goal in a temp variable
	int goal = Integer.parseInt(st.nextToken().trim());

	// store the number of moves in a temp variable
	int numMoves = Integer.parseInt(st.nextToken().trim());
	
	// get the first token of the username. 
	userName = st.nextToken();
	
	// if there are remaining elements.
	if(st.hasMoreElements()) {
	    // put the remaining tokens of the username in the string.
	    for(curToken = st.nextToken(); st.hasMoreElements(); 
		curToken = st.nextToken() ) {
			//System.out.println("cur token: " + curToken + "\n" + 
			//		     "User name: " + userName);
		userName = userName + " " + curToken;
	    }
	    
	    userName = userName + " " + curToken;
	    
	}

	frmtdStat = dateString + " " + userName + " " + goal + " " + numMoves;
	// concatonate timestamp to beginning of stats entry.	  
	//System.out.println("new entry: " + frmtdStat);

	return frmtdStat;
	
    }
    /** 
     * connectToServer assumes the client is in an online state. this methd
     * attempts to open a socke to the server.
     **/
    private boolean connectToServer() {
	int MAXPORTS = 6690;
	boolean isPortOpened = false;  // do we have access to the port?
	int portNum = 6666;
	String hostName = "";

	// if we're offline, do nothing.
	if(game.getOnlineState() == game.OFFLINE) { return false; }

	while(isPortOpened == false) {
	    try { 
			//System.out.println("server hostname: " + game.getHostName());
		// open a connection to the stats  server
		sockToServer = new Socket(game.getHostName().trim(), 
						 portNum);
		isPortOpened = true; // we're connected

		// set up readers and writers
		serverInput = new BufferedReader(
			      new InputStreamReader(
			      sockToServer.getInputStream()));
		serverOutput = new PrintWriter(
                               sockToServer.getOutputStream());
	    }

	    catch(IOException e) {
			//System.out.println("Could not connect on port: " + portNum);
		portNum = portNum + 2; 
		//System.out.println("Trying " + portNum);
	    }

	    // if we tried too many ports, return;
	    if(portNum > MAXPORTS) {
		GuiWindow gui = game.getGui();
		gui.updateChatWindow("Network Error: Could not connect to " +
				     "stats server.");
		sockToServer = null;
		return false;
	    }    
	}
	return isPortOpened;
    } // end of buildTextArea method

    private void disconnectFromServer() {
	if(sockToServer != null) {
	    try { 
		sockToServer.close();
		sockToServer = null;
	    }
	    catch (IOException e) {}
	}
    }
}
    class StatsPanel extends JPanel {
    
    StatsPanel(){
	setPreferredSize(new Dimension(20, 50));
	this.setBackground(ConnectN.GUICOLOR);
    }
    
    public void paintComponent(Graphics g){
	Graphics2D gfx = (Graphics2D)g;
	gfx.setColor(Color.black);
	gfx.fillRect(0,0,this.getWidth(),this.getHeight());
	Font font;
	int font_size = 12;
	int x = 20;
	int y = 20;
	int line_spacing = 25;
	int w;
	int speed = 1;
	for (int i = 0; i < 20; i++) {
	    font_size = 20 + i;
	    gfx.setFont(new Font("Helvetica", Font.PLAIN, font_size));
	    w = (gfx.getFontMetrics()).stringWidth("Winners");
	    gfx.setColor(new Color(65 + i * 10, 0, 0));
	    gfx.drawString("Winners", (this.getWidth() - w) / 2, 
			   ShiftSouth(y, speed * i));        
	}       			
    }
		
    int ShiftNorth(int p, int distance) {
	return (p - distance);
    }
    
    int ShiftSouth(int p, int distance) {
	return (p + distance);
    }
    
    int ShiftEast(int p, int distance) {
	return (p + distance);
    }
    
    int ShiftWest(int p, int distance) {
	return (p - distance);
    }    
   
}
